// SettingsDialog.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"   	// main symbols
#include "systrayplay.h"
#include "SettingsDialog.h"
#include "MainWnd.h"
#include "SeekDialog.h"
#include "MainHelper.h"
#include "TestPropertyPage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CPtrArray listPrefs;

/////////////////////////////////////////////////////////////////////////////
// CSettingsDialog dialog


CSettingsDialog::CSettingsDialog(UINT id, CWnd* pParent /*=NULL*/) : CDialog(id, pParent)
{
}

CSettingsDialog::~CSettingsDialog()
{
	CPropertyPage* myPage;
	if (listPropertyPage.GetSize() != 0)
		do
		{
			myPage = (CPropertyPage *)listPropertyPage[listPropertyPage.GetSize() - 1];
			delete myPage;
			myPage = NULL;
			listPropertyPage.SetSize(listPropertyPage.GetSize() - 1);
		}
		while (listPropertyPage.GetSize() != 0);
	SettingsDialog = NULL;
}

IMPLEMENT_MENUXP(CSettingsDialog, CDialog);

BEGIN_MESSAGE_MAP(CSettingsDialog, CDialog)
	ON_WM_CLOSE()
	ON_MENUXP_MESSAGES()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSettingsDialog message handlers

BOOL CSettingsDialog::DestroyWindow()
{
	m_TabSetting.DestroyWindow();
	return CDialog::DestroyWindow();
}

void CSettingsDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	CMenuXP::SetXPLookNFeel(this, pMainFrame->XPLookNFeel);
	DDX_Control(pDX, IDOK, m_BtnOk);
	DDX_Control(pDX, IDCANCEL, m_BtnCancel);
}

CSettingsDialog* CSettingsDialog::getInstance(UINT id, CWnd* pParent)
{
	if (SettingsDialog != NULL && SettingsDialog->m_hWnd != NULL)
		SettingsDialog->DestroyWindow();
	if (SettingsDialog != NULL && SettingsDialog->m_hWnd == NULL)
	{
		delete SettingsDialog;
		SettingsDialog = NULL;
	}
	if (SettingsDialog == NULL)
	{
		SettingsDialog = new CSettingsDialog(id, pParent);
		SettingsDialog->Create(id, pParent);
	}
	else
		SettingsDialog->SetFocus();
	return SettingsDialog;
}

IMPLEMENT_DYNAMIC(CSettingsDialog, CDialog)

BOOL CSettingsDialog::OnInitDialog()
{
	CDialog::OnInitDialog();
	int i = 0;

	m_Helper.translateWindowText(this, IDD_SETTINGS);
	m_Helper.translateDlgItemText(this, IDOK);
	m_Helper.translateDlgItemText(this, IDCANCEL);

	m_TabSetting.EnableStackedTabs(FALSE);
	m_TabSetting.AddPage(&m_SettingsDialogGeneral);
	m_TabSetting.AddPage(&m_SettingsDialogMouseClicks);
	m_TabSetting.AddPage(&m_SettingsDialogApparence);
	m_TabSetting.AddPage(&m_SettingsDialogApparenceTooltip);
	m_TabSetting.AddPage(&m_SettingsDialogInput);
	m_TabSetting.AddPage(&m_SettingsDialogOutput);
	m_TabSetting.AddPage(&m_SettingsDialogGen);

	//Cration des pages supplmentaires
	CTestPropertyPage* myPage = NULL;
	for (i = 0; i < listPrefs.GetSize(); i++)
	{
		prefsDlgRec* prefs = (prefsDlgRec*)listPrefs[i];
		try
		{
			myPage = new CTestPropertyPage(prefs, prefs->dlgID);
			listPropertyPage.SetSize(listPropertyPage.GetSize() + 1);
			listPropertyPage[listPropertyPage.GetSize() - 1] = myPage;
			m_TabSetting.AddPage(myPage);
			myPage->InstallHook();
		}
		catch (...)
		{
			listPrefs.RemoveAt(i);
		}
	}   
	m_TabSetting.SetTreeViewMode(TRUE, TRUE);
	m_TabSetting.SetTreeWidth(200);
	m_TabSetting.Create(this, WS_CHILD | WS_VISIBLE | WS_TABSTOP);
	m_TabSetting.ModifyStyleEx(0, WS_EX_CONTROLPARENT);

	CRect rcSheet, rcPage, rectTree;
	GetWindowRect(&rcSheet);
	ScreenToClient(&rcSheet);

	TC_ITEM item;
	CString buf;

	item.mask = TCIF_TEXT;

	m_TabSetting.SetActivePage(0);
	buf = m_Helper.translateWindowTextString(&m_SettingsDialogGeneral, IDD_SETTINGS_GENERAL);
	item.pszText = (LPTSTR)buf.GetString();
	m_TabSetting.GetTabControl()->SetItem(0, &item);
	m_TabSetting.GetActivePage()->GetClientRect(rcPage);
	if (rcPage.Width() > rcSheet.Width())
		rcSheet.right += (rcPage.Width() - rcSheet.Width());
	if (rcPage.Height() > rcSheet.Height())
		rcSheet.bottom += (rcPage.Height() - rcSheet.Height());

	m_TabSetting.SetActivePage(1);
	buf = m_Helper.translateWindowTextString(&m_SettingsDialogMouseClicks, IDD_SETTINGS_MOUSE_CLICKS);
	item.pszText = (LPTSTR)buf.GetString();
	m_TabSetting.GetTabControl()->SetItem(1, &item);
	m_TabSetting.GetActivePage()->GetClientRect(rcPage);
	if (rcPage.Width() > rcSheet.Width())
		rcSheet.right += (rcPage.Width() - rcSheet.Width());
	if (rcPage.Height() > rcSheet.Height())
		rcSheet.bottom += (rcPage.Height() - rcSheet.Height());

	m_TabSetting.SetActivePage(2);
	buf = m_Helper.translateWindowTextString(&m_SettingsDialogApparence, IDD_SETTINGS_APPARENCE);
	item.pszText = (LPTSTR)buf.GetString();
	m_TabSetting.GetTabControl()->SetItem(2, &item);
	m_TabSetting.GetActivePage()->GetClientRect(rcPage);
	if (rcPage.Width() > rcSheet.Width())
		rcSheet.right += (rcPage.Width() - rcSheet.Width());
	if (rcPage.Height() > rcSheet.Height())
		rcSheet.bottom += (rcPage.Height() - rcSheet.Height());

	m_TabSetting.SetActivePage(3);
	buf = m_Helper.translateWindowTextString(&m_SettingsDialogApparenceTooltip, IDD_SETTINGS_APPARENCE_TOOLTIP);
	item.pszText = (LPTSTR)buf.GetString();
	m_TabSetting.GetTabControl()->SetItem(3, &item);
	m_TabSetting.GetActivePage()->GetClientRect(rcPage);
	if (rcPage.Width() > rcSheet.Width())
		rcSheet.right += (rcPage.Width() - rcSheet.Width());
	if (rcPage.Height() > rcSheet.Height())
		rcSheet.bottom += (rcPage.Height() - rcSheet.Height());

	m_TabSetting.SetActivePage(4);
	buf = m_Helper.translateWindowTextString(&m_SettingsDialogInput, IDD_SETTINGS_INPUT);
	item.pszText = (LPTSTR)buf.GetString();
	m_TabSetting.GetTabControl()->SetItem(4, &item);
	if (rcPage.Width() > rcSheet.Width())
		rcSheet.right += (rcPage.Width() - rcSheet.Width());
	if (rcPage.Height() > rcSheet.Height())
		rcSheet.bottom += (rcPage.Height() - rcSheet.Height());

	m_TabSetting.SetActivePage(5);
	buf = m_Helper.translateWindowTextString(&m_SettingsDialogOutput, IDD_SETTINGS_OUTPUT);
	item.pszText = (LPTSTR)buf.GetString();
	m_TabSetting.GetTabControl()->SetItem(5, &item);
	if (rcPage.Width() > rcSheet.Width())
		rcSheet.right += (rcPage.Width() - rcSheet.Width());
	if (rcPage.Height() > rcSheet.Height())
		rcSheet.bottom += (rcPage.Height() - rcSheet.Height());

	m_TabSetting.SetActivePage(6);
	buf = m_Helper.translateWindowTextString(&m_SettingsDialogGen, IDD_SETTINGS_GEN);
	item.pszText = (LPTSTR)buf.GetString();
	m_TabSetting.GetTabControl()->SetItem(6, &item);
	if (rcPage.Width() > rcSheet.Width())
		rcSheet.right += (rcPage.Width() - rcSheet.Width());
	if (rcPage.Height() > rcSheet.Height())
		rcSheet.bottom += (rcPage.Height() - rcSheet.Height());

	for (i = 0; i < listPrefs.GetSize(); i++)
	{
		prefsDlgRec* prefs = (prefsDlgRec*)listPrefs[i];
		CString tmp2;
		if (prefs->where == 0)
		{
			buf = m_Helper.translateWindowTextString(&m_SettingsDialogGeneral, IDD_SETTINGS_GEN);
			tmp2.Format(_T("%s::"), buf);
		}
		buf.Format(_T("%s%s"), tmp2, prefs->name);
		item.pszText = (LPTSTR)buf.GetString();
		m_TabSetting.GetTabControl()->SetItem(7 + i, &item);
		m_TabSetting.GetActivePage()->GetClientRect(rcPage);
		if (rcPage.Width() > rcSheet.Width())
			rcSheet.right += (rcPage.Width() - rcSheet.Width());
		if (rcPage.Height() > rcSheet.Height())
			rcSheet.bottom += (rcPage.Height() - rcSheet.Height());
	}   

	m_TabSetting.RefillPageTree();

	m_TabSetting.SetActivePage(0);
	CRect posOk;
	m_BtnOk.GetClientRect(posOk);
	posOk.right = 70;
	posOk.bottom = 25;
	m_BtnOk.SetWindowPos(NULL, rcSheet.Width() + 40 - 2 * posOk.Width(), rcSheet.bottom - 10, posOk.Width(), posOk.Height(), SWP_NOZORDER | SWP_NOACTIVATE);
	CRect posCancel;
	m_BtnCancel.GetClientRect(posCancel);
	posCancel.right = 70;
	posCancel.bottom = 25;
	m_BtnCancel.SetWindowPos(NULL, rcSheet.Width() - 20, rcSheet.bottom - 10, posCancel.Width(), posCancel.Height(), SWP_NOZORDER | SWP_NOACTIVATE);
	SetWindowPos(NULL, -1, -1, rcSheet.Width() + 65, rcSheet.Height() + posOk.Height(), SWP_SHOWWINDOW | SWP_NOMOVE);
	m_TabSetting.GetTabControl()->ShowWindow(SW_HIDE);
	SetIcon(pMainFrame->StpIcon, TRUE);
	SetIcon(pMainFrame->StpIconSm, FALSE);

	UpdateData(false);

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void CSettingsDialog::SetActivePage(int index)
{
	/*
	** Winamp Pro   		20
	** General Preferences  0
	** File Types   		1
	** Playlist 			23
	** Titles   			21
	** Video				24
	** Skins				40
	** Classic Skins		22
	** Plugins  			30
	** Input				31
	** Output   			32
	** Visualisation		33
	** DSP/Effect   		34
	** General Purpose  	35
	*/
	switch (index)
	{
	case 20:
	case 0:
	case 1:
	case 23:
	case 21:
	case 24:
	case 33:
	case 34:
		m_TabSetting.SetActivePage(0);
		break;
	case 40:
	case 22:
		m_TabSetting.SetActivePage(2);
		break;
	case 30:
	case 31:
		m_TabSetting.SetActivePage(4);
		break;
	case 32:
		m_TabSetting.SetActivePage(5);
		break;
	case 35:
		m_TabSetting.SetActivePage(6);
		break;
	default:
		if (index >= 60)
			for (int i = 0; i < listPropertyPage.GetSize(); i++)
			{
				CTestPropertyPage* page = (CTestPropertyPage*)listPropertyPage[i];
				if (page->PageDataBackup->dlgID == index)
				{
					m_TabSetting.SetActivePage(7 + i);
					break;
				}
			}
	}
}
void CSettingsDialog::SetActivePage(prefsDlgRec* iPrefs)
{
	for (int i = 0; i < listPropertyPage.GetSize(); i++)
	{
		CTestPropertyPage* page = (CTestPropertyPage*)listPropertyPage[i];
		if (page->PageDataBackup->proc == iPrefs->proc)
		{
			m_TabSetting.SetActivePage(7 + i);
			break;
		}
	}
}

void CSettingsDialog::OnOK()
{
	UpdateData(TRUE);
	m_TabSetting.PressButton(PSBTN_APPLYNOW);
	CDialog::OnOK();
	if (pMainFrame->needToRestart)
	{
		ShellExecute(NULL, _T("open"), pMainFrame->exename, _T("-restart"), NULL, SW_SHOWNORMAL);
		MessageBox(m_Helper.MyLoadString(IDS_UPDATEOK), _T("STP"), MB_ICONINFORMATION | MB_OK);
		pMainFrame->OnPopupMenuExit();
	}
	if (SettingsDialog != NULL && SettingsDialog->m_hWnd != NULL)
		SettingsDialog->DestroyWindow();
	if (SettingsDialog != NULL)
		delete SettingsDialog;
	SettingsDialog = NULL;
}

void CSettingsDialog::OnCancel()
{
	m_TabSetting.PressButton(PSBTN_CANCEL);
	CDialog::OnCancel();
	if (SettingsDialog != NULL && SettingsDialog->m_hWnd != NULL)
		SettingsDialog->DestroyWindow();
	if (SettingsDialog != NULL)
		delete SettingsDialog;
	SettingsDialog = NULL;
}
